"""
configure and run the download of network data
"""

# import python modules
import time
import datetime
import traceback
import numpy as np
import os
import multiprocessing
from multiprocessing import set_start_method
import itertools
import importlib
import subprocess


def download_network(network, dl_data, dl_metadata, download_params, version, mode):

    dl_start_time = time.time()
    max_time_per_dl = download_params[network]['max_time_dl']
    n_max_tries = download_params[network]['max_number_tries']
    dl_module = importlib.import_module(network+'_download') # import python script


    if dl_data == True:
        print("Started downloading data of {}".format(network))
        try:
            dl_module.download_data(mode, version, n_max_tries, max_time_per_dl)
            dl_mid_time = time.time()

            # copy data to gpfs
            if mode == "nrt":
                version = mode
            else:
                version = version

            subprocess.run(['./rsync_data.sh', network, version])

            dl_end_time = time.time()

            dl_time = np.round((dl_mid_time - dl_start_time)/60., 3)
            rsync_time = np.round((dl_end_time - dl_mid_time)/60., 3)
            total_dl_time = np.round((dl_end_time - dl_start_time)/60., 3)

            return '{} successful -- total: {} min +++ download: {} min +++ rsync: {} min \n-------------------------------------------------------------------------------------------------------------\n'.format(network, total_dl_time, dl_time, rsync_time)

        except Exception as e:
            dl_end_time = time.time()
            total_dl_time = np.round((dl_end_time - dl_start_time)/60., 3)
            print(e)
            return '{} unsuccessful -- total: {} min +++ download: {} min +++ rsync: {} min \n'.format(network, total_dl_time, dl_time, rsync_time)+str(traceback.format_exc())+'\n-------------------------------------------------------------------------------------------------------------\n'


    if dl_metadata == True:
        print("Started downloading metadata of {}".format(network))
        try:
            dl_module.download_metadata(n_max_tries, max_time_per_dl)

            dl_end_time = time.time()
            total_dl_time = np.round((dl_end_time - dl_start_time)/60., 3)
            return '{} successful -- duration: {} minutes \n-------------------------------------------------------------------------------------------------------------\n'.format(network, total_dl_time)
        
        except Exception as e:
            dl_end_time = time.time()
            total_dl_time = np.round((dl_end_time - dl_start_time)/60., 3)
            print(e)
            return '{} unsuccessful -- duration {} minutes \n'.format(network, total_dl_time)+str(traceback.format_exc())+'\n-------------------------------------------------------------------------------------------------------------\n'




if __name__ == "__main__":

    # modes: 'all': download all the available files
    #        'nrt': download most recent file
    mode = 'nrt'
    print("Mode: {}".format(mode))

    # ghost version
    version = '1.6'
    print("Ghost version: {}".format(version))

    # maximum number of downloads attempts
    n_max_tries = 3

    # maximum time one download attempt should take per network
    download_params = {'US_EPA_AQS':                {'max_time_dl': 5,  'max_number_tries': 3},
                        'US_EPA_CASTNET':           {'max_time_dl': 10, 'max_number_tries': 3},
                        'EEA_AQ_eReporting':        {'max_time_dl': 5,  'max_number_tries': 3},
                        'AERONETv3':                {'max_time_dl': 1,  'max_number_tries': 3},
                        'BJMEMC':                   {'max_time_dl': 3,  'max_number_tries': 3},
                        'CNEMC':                    {'max_time_dl': 3,  'max_number_tries': 3},
                        'CANADA_NAPS':              {'max_time_dl': 5,  'max_number_tries': 3},
                        'CAPMoN':                   {'max_time_dl': 5,  'max_number_tries': 3},
                        'US_NADP_AMNet':            {'max_time_dl': 10, 'max_number_tries': 3},
                        'US_NADP_AMoN':             {'max_time_dl': 7,  'max_number_tries': 3},
                        'MEXICO_CDMX':              {'max_time_dl': 10, 'max_number_tries': 3},
                        'NOAA_ISD':                 {'max_time_dl': 15, 'max_number_tries': 3},
                        'MITECO':                   {'max_time_dl': 10, 'max_number_tries': 3},
                        'EANET':                    {'max_time_dl': 5,  'max_number_tries': 3},
                        'CHILE_SINCA':              {'max_time_dl': 30, 'max_number_tries': 3},
                        'EEA_AQ_eReporting':        {'max_time_dl': 20, 'max_number_tries': 3},
                        'JAPAN_NIES':               {'max_time_dl': 60, 'max_number_tries': 3},
                        'WMO_WDCGG':                {'max_time_dl': 60, 'max_number_tries': 3},
                        'UK_AIR':                   {'max_time_dl': 60, 'max_number_tries': 3},
                        'US_EPA_AirNow_DOS':        {'max_time_dl': 10, 'max_number_tries': 3}}


    # download data
    dl_data = True

    # download meta data too?
    dl_metadata = False

    # networks you want to download
    #networks = ['CHILE_SINCA']
    #networks = ['US_NADP_AMNet', 'US_NADP_AMoN', 'US_EPA_CASTNET', 'BJMEMC', 'CNEMC', 'US_EPA_AirNow_DOS', 'WMO_WDCGG', 'JAPAN_NIES']
    networks = ['CHILE_SINCA']

    # download all networks
    #networks = 'all'
    if networks == 'all':
        if mode == 'all':
            networks = ['US_EPA_AirNow_DOS', 
                        'US_EPA_CASTNET', 
                        'BJMEMC', 
                        'CNEMC', 
                        'WMO_WDCGG', 
                        'CAPMoN', 
                        'EANET', 
                        'CANADA_NAPS', 
                        'CHILE_SINCA', 
                        'US_NADP_AMNet', 
                        'US_NADP_AMoN', 
                        'MEXICO_CDMX', 
                        'JAPAN_NIES', 
                        'MITECO', 
                        'UK_AIR', 
                        'NOAA_ISD', 
                        'US_EPA_AQS', 
                        'AERONETv3', 
                        'EEA_AQ_eReporting', 
                        'EBAS']
            
        elif mode == 'nrt':
            networks = ['US_EPA_AirNow_DOS', # download current month, response error 300; monthly files
                        'US_EPA_CASTNET', # download 2024; error occured randomly: requests.exceptions.ConnectionError: HTTPSConnectionPool(host='gaftp.epa.gov', port=443): Max retries exceeded with url: /castnet/CASTNET_Outgoing/data/ozone_2024.zip (Caused by NewConnectionError('<urllib3.connection.HTTPSConnection object at 0x7f54f0e7f3d0>: Failed to establish a new connection: [Errno -2] Name or service not known'))
                        'BJMEMC', # went fine, download only last day
                        'CNEMC', # went fine, download only last day
                        'WMO_WDCGG', # corrected 
                        'CHILE_SINCA', # ???, download last 30 days
                        'JAPAN_NIES', # went fine, download current month
                        'UK_AIR', # went fine, download 2024; only one stream 'auto' -> corrected, now all streams
                        'NOAA_ISD', # went fine, download 2024
                        'AERONETv3', # went fine, download only last day
                        'EEA_AQ_eReporting'] # went fine, E2a stream from 2023-01-01 until now ### new download: daily files

    
    # create logfile
    dl_logfile = open('/esarchive/scratch/rgrodofz/software/ghost/GHOST/download_scripts/log_files/log_{}.out'.format(datetime.datetime.now().strftime('%Y%m%d-%H%M%S')),"w")
    dl_logfile.write('**************************************************************************************************************\n')        
    dl_logfile.write('BEGINNING DOWNLOADING DATA {}, GHOST version {}\n'.format(mode, version))
    dl_logfile.write('**************************************************************************************************************\n')

    # start downloading in parallel
    dl_start_time = time.time()
    pool = multiprocessing.Pool(1) # two CPUs are being used
    result = pool.starmap(download_network, zip(networks, itertools.repeat(dl_data), itertools.repeat(dl_metadata), itertools.repeat(download_params), itertools.repeat(version), itertools.repeat(mode)))
    dl_logfile.write(''.join(result))

    pool.close()
    pool.join()

    dl_end_time = time.time()
    dl_time_tot = np.round((dl_end_time - dl_start_time)/60., 3)
    dl_logfile.write('Total duration: {} minutes'.format(dl_time_tot))
    dl_logfile.close()


    # run the desired networks
    # for network in networks:

    #     dl_start_time = time.time()
    #     max_time_per_dl = download_params[network]['max_time_dl']
    #     n_max_tries = download_params[network]['max_number_tries']
    #     dl_module = importlib.import_module(network+'_download') # import python script

    #     if dl_data == True:
    #         print("Started downloading data of {}".format(network))
    #         try:
    #             dl_module.download_data(mode, version, n_max_tries, max_time_per_dl)

    #             # copy data to gpfs
    #             #os.system('rsync -arvzh /esarchive/obs/ghost/{}/ /gpfs/projects/bsc32/AC_cache/obs/ghost/{} > rsync_gpfs.txt'.format(network_name, network_name))

    #             dl_end_time = time.time()
    #             total_dl_time = (dl_end_time - dl_start_time)/60.
    #             #dl_logfile.write('{} successful -- duration: {} minutes'.format(network_name, total_dl_time))
    #             #dl_logfile.write('\n---------------------------------------------------------------------------------------------------\n')
                

    #         except Exception as e:
    #             print(e)
    #             dl_end_time = time.time()
    #             total_dl_time = np.round((dl_end_time - dl_start_time)/60., 3)
    #             print("failed {}".format(total_dl_time))
    #             #dl_logfile.write('{} unsuccessful -- duration {} minutes \n'.format(network_name, total_dl_time)+str(traceback.format_exc()))
    #             #dl_logfile.write('\n---------------------------------------------------------------------------------------------------\n')


    #     if dl_metadata == True:
    #         print("Started downloading metadata of {}".format(network))
    #         dl_module.download_metadata(n_max_tries, max_time_per_dl)

